﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DWZB_Agv_Agreement
{
    /// <summary>
    /// Copyright20181102 (C) sunyalong
    /// 数据访问抽象基础类(基于SQLServer)
    /// 允许修改、添加满足自己项目的需要。
    /// 添加、修改后请详细注释。违者会强制删除不予采用。
    /// </summary>
    public abstract class DbHelperSQLServer
    {
        public static string connectionString = DbConfigureConnectionString.GetConnectionString("LeadSqlServer");

        #region 公用方法
        /// <summary>
        /// 判断是否存在某表的某个字段
        /// </summary>
        /// <param name="tableName">表名称</param>
        /// <param name="columnName">列名称</param>
        /// <returns>是否存在</returns>
        public static bool ColumnExists(string tableName, string columnName)
        {
            string sql = "select count(1) from syscolumns where [id]=object_id('" + tableName + "') and [name]='" + columnName + "'";
            object res = GetSingle(sql);
            if (res == null)
            {
                return false;
            }
            return Convert.ToInt32(res) > 0;
        }
        /// <summary>
        /// 表是否存在
        /// </summary>
        /// <param name="TableName">数据表名称</param>
        /// <returns></returns>
        public static bool TabExists(string TableName)
        {
            string strsql = "select count(*) from sysobjects where id = object_id(N'[" + TableName + "]') and OBJECTPROPERTY(id, N'IsUserTable') = 1";
            object obj = GetSingle(strsql);
            int cmdresult;
            if ((Equals(obj, null)) || (Equals(obj, DBNull.Value)))
            {
                cmdresult = 0;
            }
            else
            {
                cmdresult = int.Parse(obj.ToString());
            }
            if (cmdresult == 0)
            {
                return false;
            }
            else
            {
                return true;
            }
        }
        #endregion

        #region  执行简单SQL语句

        /// <summary>
        /// 执行SQL语句，返回影响的记录数
        /// </summary>
        /// <param name="SQLString">SQL语句</param>
        /// <returns>影响的记录数</returns>
        public static int ExecuteSql(string SQLString)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(SQLString, connection))
                {
                    try
                    {
                        connection.Open();
                        int rows = cmd.ExecuteNonQuery();
                        return rows;
                    }
                    catch (SqlException e)
                    {
                        connection.Close();
                        LogHelper.WriteLog($"执行SQL语句发生异常：【{e.Message}】！SQL：【{SQLString}】", LogTypeEnum.DbErr);
                        return 0;
                    }
                }
            }
        }
        /// <summary>
        /// 执行SQL语句，返回影响的记录数。（可配置超时时间）
        /// </summary>
        /// <param name="SQLString">SQL语句</param>
        /// <param name="Times">超时时间设置（单位：秒）默认值30</param>
        /// <returns></returns>
        public static int ExecuteSqlByTime(string SQLString, int Times)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(SQLString, connection))
                {
                    try
                    {
                        connection.Open();
                        cmd.CommandTimeout = Times;
                        int rows = cmd.ExecuteNonQuery();
                        return rows;
                    }
                    catch (SqlException e)
                    {
                        connection.Close();
                        LogHelper.WriteLog($"执行SQL语句发生异常：【{e.Message}】！SQL：【{SQLString}】超时时间：【{Times}】", LogTypeEnum.DbErr);
                        return 0;
                    }
                }
            }
        }

        /// <summary>
        /// 执行多条SQL语句，实现数据库事务。
        /// </summary>
        /// <param name="SQLStringList">多条SQL语句List集合</param>		
        public static int ExecuteSqlTran(List<string> SQLStringList)
        {
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                conn.Open();
                using (SqlCommand cmd = new SqlCommand())
                {
                    cmd.Connection = conn;
                    SqlTransaction tx = conn.BeginTransaction();
                    cmd.Transaction = tx;
                    try
                    {
                        int count = 0;
                        for (int n = 0; n < SQLStringList.Count; n++)
                        {
                            string strsql = SQLStringList[n];
                            if (strsql.Trim().Length > 1)
                            {
                                cmd.CommandText = strsql;
                                count += cmd.ExecuteNonQuery();
                            }
                        }
                        tx.Commit();
                        return count;
                    }
                    catch (Exception ex)
                    {
                        tx.Rollback();
                        string errSql = string.Empty;
                        for (int n = 0; n < SQLStringList.Count; n++)
                        {
                            errSql += $"【{SQLStringList[n]}】";
                        }
                        LogHelper.WriteLog($"执行SQL语句发生异常：【{ex.Message}】！SQL语句：{errSql}", LogTypeEnum.DbErr);
                        return 0;
                    }
                }
            }
        }
        /// <summary>
        /// 执行带一个特殊参数的的SQL语句。
        /// </summary>
        /// <param name="SQLString">SQL语句</param>
        /// <param name="content">参数内容,比如一个字段是格式复杂的文章，有特殊符号，可以通过这个方式添加参数</param>
        /// <returns>影响的记录数</returns>
        public static int ExecuteSql(string SQLString, string content)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(SQLString, connection))
                {
                    SqlParameter myParameter = new SqlParameter("@content", SqlDbType.NText)
                    {
                        Value = content
                    };
                    cmd.Parameters.Add(myParameter);
                    try
                    {
                        connection.Open();
                        int rows = cmd.ExecuteNonQuery();
                        return rows;
                    }
                    catch (SqlException e)
                    {
                        LogHelper.WriteLog($"执行SQL语句发生异常：【{e.Message}】！SQL：【{SQLString}】特殊参数：【{content}】", LogTypeEnum.DbErr);
                        return 0;
                    }
                    finally
                    {
                        cmd.Dispose();
                        connection.Close();
                    }
                }
            }
        }
        /// <summary>
        /// 向数据库里插入图像格式的字段(和上面情况类似的另一种实例)
        /// </summary>
        /// <param name="strSQL">SQL语句</param>
        /// <param name="fs">图像字节,数据库的字段类型为image的情况</param>
        /// <returns>影响的记录数</returns>
        public static int ExecuteSqlInsertImg(string strSQL, byte[] fs)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(strSQL, connection))
                {
                    SqlParameter myParameter = new SqlParameter("@fs", SqlDbType.Image)
                    {
                        Value = fs
                    };
                    cmd.Parameters.Add(myParameter);
                    try
                    {
                        connection.Open();
                        int rows = cmd.ExecuteNonQuery();
                        return rows;
                    }
                    catch (SqlException e)
                    {
                        string bytStr = string.Empty;
                        foreach (byte item in fs)
                        {
                            bytStr += $"【{item}】";
                        }
                        LogHelper.WriteLog($"向数据库里插入图像格式的字段发生异常：【{e.Message}】！SQL：【{strSQL}】图像字节：【{bytStr}】", LogTypeEnum.DbErr);
                        return 0;
                    }
                    finally
                    {
                        cmd.Dispose();
                        connection.Close();
                    }
                }
            }
        }

        /// <summary>
        /// 执行一条计算查询结果语句，返回查询结果（object）。
        /// </summary>
        /// <param name="SQLString">计算查询结果sql语句</param>
        /// <returns>查询结果（object）</returns>
        public static object GetSingle(string SQLString)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(SQLString, connection))
                {
                    try
                    {
                        connection.Open();
                        object obj = cmd.ExecuteScalar();
                        if ((Equals(obj, null)) || (Equals(obj, DBNull.Value)))
                        {
                            return null;
                        }
                        else
                        {
                            return obj;
                        }
                    }
                    catch (SqlException e)
                    {
                        connection.Close();
                        LogHelper.WriteLog($"执行一条计算查询结果的语句发生异常：【{e.Message}】！SQL：【{SQLString}】", LogTypeEnum.DbErr);
                        return null;
                    }
                }
            }
        }
        /// <summary>
        /// 执行一条计算查询结果语句，返回查询结果（object）。
        /// </summary>
        /// <param name="SQLString">计算查询结果sql语句</param>
        /// <param name="Times">超时时间设置（单位：秒）默认值30</param>
        /// <returns></returns>
        public static object GetSingle(string SQLString, int Times)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(SQLString, connection))
                {
                    try
                    {
                        connection.Open();
                        cmd.CommandTimeout = Times;
                        object obj = cmd.ExecuteScalar();
                        if ((Equals(obj, null)) || (Equals(obj, DBNull.Value)))
                        {
                            return null;
                        }
                        else
                        {
                            return obj;
                        }
                    }
                    catch (SqlException e)
                    {
                        connection.Close();
                        LogHelper.WriteLog($"执行一条计算查询结果的语句发生异常：【{e.Message}】！SQL：【{SQLString}】超时时间：【{Times}】", LogTypeEnum.DbErr);
                        return null;
                    }
                }
            }
        }
        /// <summary>
        /// 执行查询语句，返回SqlDataReader ( 注意：调用该方法后，一定要对SqlDataReader进行Close )
        /// </summary>
        /// <param name="strSQL">查询语句</param>
        /// <returns>SqlDataReader</returns>
        public static SqlDataReader ExecuteReader(string strSQL)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand(strSQL, connection))
                {
                    try
                    {
                        connection.Open();
                        SqlDataReader myReader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                        return myReader;
                    }
                    catch (SqlException e)
                    {
                        LogHelper.WriteLog($"执行查询语句，返回SqlDataReader发生异常：【{e.Message}】！SQL：【{strSQL}】", LogTypeEnum.DbErr);
                        return null;
                    }
                }
            }

        }
        /// <summary>
        /// 执行查询语句，返回DataSet
        /// </summary>
        /// <param name="SQLString">查询sql语句</param>
        /// <returns>DataSet</returns>
        public static DataSet Query(string SQLString)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                DataSet ds = new DataSet();
                try
                {
                    connection.Open();
                    SqlDataAdapter command = new SqlDataAdapter(SQLString, connection);
                    command.Fill(ds, "ds");
                }
                catch (SqlException ex)
                {
                    LogHelper.WriteLog($"执行查询语句，返回DataSet发生异常：【{ex.Message}】！SQL：【{SQLString}】", LogTypeEnum.DbErr);
                }
                return ds;
            }
        }
        /// <summary>
        /// 执行查询语句，返回DataSet
        /// </summary>
        /// <param name="SQLString">查询sql语句</param>
        /// <param name="Times">超时时间设置（单位：秒）默认值30</param>
        /// <returns></returns>
        public static DataSet Query(string SQLString, int Times)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                DataSet ds = new DataSet();
                try
                {
                    connection.Open();
                    SqlDataAdapter command = new SqlDataAdapter(SQLString, connection);
                    command.SelectCommand.CommandTimeout = Times;
                    command.Fill(ds, "ds");
                }
                catch (SqlException ex)
                {
                    LogHelper.WriteLog($"执行查询语句，返回DataSet发生异常：【{ex.Message}】！SQL：【{SQLString}】超时时间：【{Times}】", LogTypeEnum.DbErr);
                }
                return ds;
            }
        }



        #endregion

        #region 执行带参数的SQL语句

        /// <summary>
        /// 执行SQL语句，返回影响的记录数
        /// </summary>
        /// <param name="SQLString">SQL语句</param>
        /// <param name="cmdParms">参数集合</param>
        /// <returns>影响的记录数</returns>
        public static int ExecuteSql(string SQLString, params SqlParameter[] cmdParms)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand())
                {
                    string retParmStr = string.Empty;
                    try
                    {
                        retParmStr = PrepareCommand(cmd, connection, null, SQLString, cmdParms);
                        int rows = cmd.ExecuteNonQuery();
                        cmd.Parameters.Clear();
                        return rows;
                    }
                    catch (SqlException ex)
                    {
                        LogHelper.WriteLog($"执行SQL语句，返回影响的记录数发生异常：【{ex.Message}】！SQL：【{SQLString}】{retParmStr}", LogTypeEnum.DbErr);
                        return 0;
                    }
                }
            }
        }


        /// <summary>
        /// 执行多条SQL语句，实现数据库事务。
        /// </summary>
        /// <param name="SQLStringList">SQL语句的哈希表（key为sql语句，value是该语句的SqlParameter[]）</param>
        /// <returns>影响的记录数</returns>
        public static int ExecuteSqlTran(Hashtable SQLStringList)
        {
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                conn.Open();
                using (SqlTransaction trans = conn.BeginTransaction())
                {
                    SqlCommand cmd = new SqlCommand();
                    try
                    {
                        int val = 0;
                        //循环
                        foreach (DictionaryEntry myDE in SQLStringList)
                        {
                            string cmdText = myDE.Key.ToString();
                            SqlParameter[] cmdParms = (SqlParameter[])myDE.Value;
                            PrepareCommand(cmd, conn, trans, cmdText, cmdParms);
                            val += cmd.ExecuteNonQuery();
                            cmd.Parameters.Clear();
                        }
                        trans.Commit();
                        return val;
                    }
                    catch (Exception ex)
                    {
                        trans.Rollback();
                        string retStr = string.Empty;
                        foreach (DictionaryEntry myDE in SQLStringList)
                        {
                            string cmdText = myDE.Key.ToString();
                            SqlParameter[] cmdParms = (SqlParameter[])myDE.Value;
                            if (cmdParms.Length > 0)
                            {
                                string paramStr = string.Empty;
                                foreach (SqlParameter item in cmdParms)
                                {
                                    paramStr += $"(参数名称：[{item.ParameterName}]参数值：[{item.Value}])";
                                }
                                retStr += $"【sql：[{cmdText}] 参数集合：{paramStr}】";
                            }
                            else
                            {
                                retStr += $"【sql：[{cmdText}]】";
                            }
                        }
                        LogHelper.WriteLog($"执行多条SQL语句，实现数据库事务发生异常：【{ex.Message}】！{retStr}", LogTypeEnum.DbErr);
                        return 0;
                    }
                }
            }
        }

        /// <summary>
        /// 执行一条计算查询结果语句，返回查询结果（object）。
        /// </summary>
        /// <param name="SQLString">计算查询结果语句</param>
        /// <returns>查询结果（object）</returns>
        public static object GetSingle(string SQLString, params SqlParameter[] cmdParms)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand())
                {
                    string retParmStr = string.Empty;
                    try
                    {
                        retParmStr = PrepareCommand(cmd, connection, null, SQLString, cmdParms);
                        object obj = cmd.ExecuteScalar();
                        cmd.Parameters.Clear();
                        if ((Equals(obj, null)) || (Equals(obj, DBNull.Value)))
                        {
                            return null;
                        }
                        else
                        {
                            return obj;
                        }
                    }
                    catch (SqlException ex)
                    {
                        LogHelper.WriteLog($"执行一条计算查询结果语句，返回查询结果（object）发生异常：【{ex.Message}】！SQL：【{SQLString}】{retParmStr}", LogTypeEnum.DbErr);
                        return 0;
                    }
                }
            }
        }

        /// <summary>
        /// 执行查询语句，返回SqlDataReader ( 注意：调用该方法后，一定要对SqlDataReader进行Close )
        /// </summary>
        /// <param name="strSQL">查询语句</param>
        /// <returns>SqlDataReader</returns>
        public static SqlDataReader ExecuteReader(string SQLString, params SqlParameter[] cmdParms)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand())
                {
                    string retParmStr = string.Empty;
                    try
                    {
                        retParmStr = PrepareCommand(cmd, connection, null, SQLString, cmdParms);
                        SqlDataReader myReader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                        cmd.Parameters.Clear();
                        return myReader;
                    }
                    catch (SqlException ex)
                    {
                        LogHelper.WriteLog($"执行查询语句，返回SqlDataReader发生异常：【{ex.Message}】！SQL：【{SQLString}】{retParmStr}", LogTypeEnum.DbErr);
                        return null;
                    }
                    finally
                    {
                        cmd.Dispose();
                        connection.Close();
                    }
                }
            }
        }

        /// <summary>
        /// 执行查询语句，返回DataSet
        /// </summary>
        /// <param name="SQLString">查询语句</param>
        /// <param name="cmdParms">参数集合</param>
        /// <returns></returns>
        public static DataSet Query(string SQLString, params SqlParameter[] cmdParms)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand cmd = new SqlCommand())
                {
                    string paramStr = string.Empty;
                    paramStr = PrepareCommand(cmd, connection, null, SQLString, cmdParms);
                    using (SqlDataAdapter da = new SqlDataAdapter(cmd))
                    {
                        DataSet ds = new DataSet();
                        try
                        {
                            da.Fill(ds, "ds");
                            cmd.Parameters.Clear();
                        }
                        catch (SqlException ex)
                        {
                            LogHelper.WriteLog($"执行查询语句，返回DataSet发生异常：【{ex.Message}】！SQL：【{SQLString}】{paramStr}", LogTypeEnum.DbErr);
                        }
                        return ds;
                    }
                }
            }
        }


        private static string PrepareCommand(SqlCommand cmd, SqlConnection conn, SqlTransaction trans, string cmdText, SqlParameter[] cmdParms)
        {
            if (conn.State != ConnectionState.Open)
                conn.Open();
            cmd.Connection = conn;
            cmd.CommandText = cmdText;
            if (trans != null)
                cmd.Transaction = trans;
            cmd.CommandType = CommandType.Text;//cmdType;
            if (cmdParms != null)
            {
                string paramStr = string.Empty;
                foreach (SqlParameter parameter in cmdParms)
                {
                    if ((parameter.Direction == ParameterDirection.InputOutput || parameter.Direction == ParameterDirection.Input) &&
                        (parameter.Value == null))
                    {
                        parameter.Value = DBNull.Value;
                    }
                    cmd.Parameters.Add(parameter);
                    paramStr += $"【参数名称：[{parameter.ParameterName}]参数值：[{parameter.Value}]】";
                }
                return paramStr;
            }
            else
            {
                return null;
            }
        }

        #endregion

        #region 存储过程操作

        /// <summary>
        /// 执行存储过程，返回SqlDataReader ( 注意：调用该方法后，一定要对SqlDataReader进行Close )
        /// </summary>
        /// <param name="storedProcName">存储过程名</param>
        /// <param name="parameters">存储过程参数</param>
        /// <returns>SqlDataReader</returns>
        public static SqlDataReader RunProcedure(string storedProcName, IDataParameter[] parameters)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                SqlDataReader returnReader;
                connection.Open();
                using (SqlCommand command = BuildQueryCommand(connection, storedProcName, parameters))
                {
                    try
                    {
                        command.CommandType = CommandType.StoredProcedure;
                        returnReader = command.ExecuteReader(CommandBehavior.CloseConnection);
                        return returnReader;
                    }
                    catch (Exception ex)
                    {
                        string retParamStr = string.Empty;
                        foreach (IDataParameter item in parameters)
                        {
                            retParamStr += $"【参数名称：[{item.ParameterName}]参数值：[{item.Value}]】";
                        }
                        LogHelper.WriteLog($"执行存储过程，返回SqlDataReader发生异常：【{ex.Message}】！存储过程：【{storedProcName}】{retParamStr}", LogTypeEnum.DbErr);
                        return null;
                    }
                }
            }
        }


        /// <summary>
        /// 执行存储过程,返回存储过程结果集DataSet中的某个表。
        /// </summary>
        /// <param name="storedProcName">存储过程名</param>
        /// <param name="parameters">存储过程参数</param>
        /// <param name="tableName">DataSet结果中的表名</param>
        /// <returns>DataSet</returns>
        public static DataSet RunProcedure(string storedProcName, IDataParameter[] parameters, string tableName)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                DataSet dataSet = new DataSet();
                connection.Open();
                using (SqlDataAdapter sqlDA = new SqlDataAdapter())
                {
                    try
                    {
                        sqlDA.SelectCommand = BuildQueryCommand(connection, storedProcName, parameters);
                        sqlDA.Fill(dataSet, tableName);
                        connection.Close();
                        return dataSet;
                    }
                    catch (Exception ex)
                    {
                        string retParamStr = string.Empty;
                        foreach (IDataParameter item in parameters)
                        {
                            retParamStr += $"【参数名称：[{item.ParameterName}]参数值：[{item.Value}]】";
                        }
                        LogHelper.WriteLog($"执行存储过程,返回存储过程结果集DataSet中的某个表发生异常：【{ex.Message}】！存储过程：【{storedProcName}】{retParamStr} 表名称：【{tableName}】", LogTypeEnum.DbErr);
                        return dataSet;
                    }
                }
            }
        }
        /// <summary>
        /// 执行存储过程,返回存储过程结果集DataSet中的某个表。
        /// </summary>
        /// <param name="storedProcName">存储过程名</param>
        /// <param name="parameters">存储过程参数</param>
        /// <param name="tableName">DataSet结果中的表名</param>
        /// <param name="Times">超时时间设置（单位：秒）默认值30</param>
        /// <returns>DataSet</returns>
        public static DataSet RunProcedure(string storedProcName, IDataParameter[] parameters, string tableName, int Times)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                DataSet dataSet = new DataSet();
                connection.Open();
                using (SqlDataAdapter sqlDA = new SqlDataAdapter())
                {
                    try
                    {
                        sqlDA.SelectCommand = BuildQueryCommand(connection, storedProcName, parameters);
                        sqlDA.SelectCommand.CommandTimeout = Times;
                        sqlDA.Fill(dataSet, tableName);
                        connection.Close();
                        return dataSet;
                    }
                    catch (Exception ex)
                    {
                        string retParamStr = string.Empty;
                        foreach (IDataParameter item in parameters)
                        {
                            retParamStr += $"【参数名称：[{item.ParameterName}]参数值：[{item.Value}]】";
                        }
                        LogHelper.WriteLog($"执行存储过程,返回存储过程结果集DataSet中的某个表发生异常：【{ex.Message}】！存储过程：【{storedProcName}】{retParamStr} 表名称：【{tableName}】超时时间：【{Times}】", LogTypeEnum.DbErr);
                        return dataSet;
                    }
                }
            }
        }


        /// <summary>
        /// 构建 SqlCommand 对象(用来返回一个结果集，而不是一个整数值)
        /// </summary>
        /// <param name="connection">数据库连接</param>
        /// <param name="storedProcName">存储过程名</param>
        /// <param name="parameters">存储过程参数</param>
        /// <returns>SqlCommand</returns>
        private static SqlCommand BuildQueryCommand(SqlConnection connection, string storedProcName, IDataParameter[] parameters)
        {
            SqlCommand command = new SqlCommand(storedProcName, connection)
            {
                CommandType = CommandType.StoredProcedure
            };
            foreach (SqlParameter parameter in parameters)
            {
                if (parameter != null)
                {
                    // 检查未分配值的输出参数,将其分配以DBNull.Value.
                    if ((parameter.Direction == ParameterDirection.InputOutput || parameter.Direction == ParameterDirection.Input) &&
                        (parameter.Value == null))
                    {
                        parameter.Value = DBNull.Value;
                    }
                    command.Parameters.Add(parameter);
                }
            }

            return command;
        }

        /// <summary>
        /// 执行存储过程，返回返回一个整数值
        /// </summary>
        /// <param name="storedProcName">存储过程名</param>
        /// <param name="parameters">存储过程参数</param>
        /// <param name="rowsAffected">影响的行数</param>
        /// <returns></returns>
        public static int RunProcedure(string storedProcName, IDataParameter[] parameters, out int rowsAffected)
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                int result;
                connection.Open();
                using (SqlCommand command = BuildIntCommand(connection, storedProcName, parameters))
                {
                    try
                    {
                        rowsAffected = command.ExecuteNonQuery();
                        result = (int)command.Parameters["ReturnValue"].Value;
                        return result;
                    }
                    catch (Exception ex)
                    {
                        string retParamStr = string.Empty;
                        foreach (IDataParameter item in parameters)
                        {
                            retParamStr += $"【参数名称：[{item.ParameterName}]参数值：[{item.Value}]】";
                        }
                        LogHelper.WriteLog($"执行存储过程,返回存储过程结果集DataSet中的某个表发生异常：【{ex.Message}】！存储过程：【{storedProcName}】{retParamStr} ", LogTypeEnum.DbErr);
                        rowsAffected = 0;
                        return 0;
                    }
                }
            }
        }

        /// <summary>
        /// 创建 SqlCommand 对象实例(用来返回一个整数值)	
        /// </summary>
        /// <param name="storedProcName">存储过程名</param>
        /// <param name="parameters">存储过程参数</param>
        /// <returns>SqlCommand 对象实例</returns>
        private static SqlCommand BuildIntCommand(SqlConnection connection, string storedProcName, IDataParameter[] parameters)
        {
            SqlCommand command = BuildQueryCommand(connection, storedProcName, parameters);
            command.Parameters.Add(new SqlParameter("ReturnValue",
                SqlDbType.Int, 4, ParameterDirection.ReturnValue,
                false, 0, 0, string.Empty, DataRowVersion.Default, null));
            return command;
        }
        #endregion

        #region 自定义方法
        /// <summary>
        /// 数据库是否连接成功
        /// </summary>
        /// <returns></returns>
        public static bool ConnStatus()
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                try
                {
                    connection.Open();
                    return true;
                }
                catch (SqlException ex)
                {
                    connection.Close();
                    LogHelper.WriteLog($"测试数据库是否连接成功发生异常：【{ex.Message}】！", LogTypeEnum.DbErr);
                    return false;
                }
            }
        }

        #endregion

    }
}
